
#ifndef KDEVPLATFORMDEBUGGER_EXPORT_H
#define KDEVPLATFORMDEBUGGER_EXPORT_H

#ifdef KDEVPLATFORMDEBUGGER_STATIC_DEFINE
#  define KDEVPLATFORMDEBUGGER_EXPORT
#  define KDEVPLATFORMDEBUGGER_NO_EXPORT
#else
#  ifndef KDEVPLATFORMDEBUGGER_EXPORT
#    ifdef KDevPlatformDebugger_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMDEBUGGER_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMDEBUGGER_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMDEBUGGER_NO_EXPORT
#    define KDEVPLATFORMDEBUGGER_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMDEBUGGER_DEPRECATED
#  define KDEVPLATFORMDEBUGGER_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMDEBUGGER_DEPRECATED_EXPORT
#  define KDEVPLATFORMDEBUGGER_DEPRECATED_EXPORT KDEVPLATFORMDEBUGGER_EXPORT KDEVPLATFORMDEBUGGER_DEPRECATED
#endif

#ifndef KDEVPLATFORMDEBUGGER_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMDEBUGGER_DEPRECATED_NO_EXPORT KDEVPLATFORMDEBUGGER_NO_EXPORT KDEVPLATFORMDEBUGGER_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMDEBUGGER_NO_DEPRECATED
#    define KDEVPLATFORMDEBUGGER_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMDEBUGGER_EXPORT_H */
