
#ifndef KDEVPLATFORMSUBLIME_EXPORT_H
#define KDEVPLATFORMSUBLIME_EXPORT_H

#ifdef KDEVPLATFORMSUBLIME_STATIC_DEFINE
#  define KDEVPLATFORMSUBLIME_EXPORT
#  define KDEVPLATFORMSUBLIME_NO_EXPORT
#else
#  ifndef KDEVPLATFORMSUBLIME_EXPORT
#    ifdef KDevPlatformSublime_EXPORTS
        /* We are building this library */
#      define KDEVPLATFORMSUBLIME_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KDEVPLATFORMSUBLIME_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KDEVPLATFORMSUBLIME_NO_EXPORT
#    define KDEVPLATFORMSUBLIME_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KDEVPLATFORMSUBLIME_DEPRECATED
#  define KDEVPLATFORMSUBLIME_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KDEVPLATFORMSUBLIME_DEPRECATED_EXPORT
#  define KDEVPLATFORMSUBLIME_DEPRECATED_EXPORT KDEVPLATFORMSUBLIME_EXPORT KDEVPLATFORMSUBLIME_DEPRECATED
#endif

#ifndef KDEVPLATFORMSUBLIME_DEPRECATED_NO_EXPORT
#  define KDEVPLATFORMSUBLIME_DEPRECATED_NO_EXPORT KDEVPLATFORMSUBLIME_NO_EXPORT KDEVPLATFORMSUBLIME_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KDEVPLATFORMSUBLIME_NO_DEPRECATED
#    define KDEVPLATFORMSUBLIME_NO_DEPRECATED
#  endif
#endif

#endif /* KDEVPLATFORMSUBLIME_EXPORT_H */
